// dateien laden / sreiben: neue filestruktur
#include <afxwin.h>         // MFC core and standard components
#include <afxext.h>
#include "ladedatei.h"
#include "makrosa.h"

// write or read element structure
// retw: 1: ok <0: file not read or written 
int getFEStruct(int inout, // 1=write;2=read;3=get-param only for space allocation
	char *file,		// filetitel
	int param[12],	// params see below
	int *nrp,		// node IDs
	float *koor,	// node koordinates	koor[][3]
	int *nel,		// elementdata nel[][nn1]
	int *layerid,		// layer-ids of the elements
	int *color,		// color-id
	int *layeridd,	// layerids -> layernamen
	int *layerixf,	// layerfarbe
	char *layername, // layernamen
	float *koorsys,	// koorsystems koor[][7][3]
	void *data)	// additional data values if any
{
	static versnr = 2;		// version of filestructure: witten to the file in param[11]
	int np = param[0];		// Number of nodes
	int ne = param[1];		// number of elements
	int nn1 = param[2];		// dimension of nel[][nn1]
	int nkoors = param[3];	// number of koorsytems
	int nww = param[4];	// length of data in bytes (spezial internal data)
	int nla = param[5];	// number of names in layeridd
	int nln = param[6];	// laenge von layername
	// typ = param[10];	// == 1 : elementstructure
	// vers = param[11];	// == versnr : version number
	int nw,nvers;

	FILE *pf;

	if(inout > 1)
		memset(param,0,12*sizeof(int));

	if(inout > 1)
		pf = fopen(file,"rb");
	else
		pf = fopen(file,"wb");

	if(!pf) 
	{
#if !defined(_MAKROSA)
		printf("file %s \ncould not be opened",file);
#endif
		return -1;
	}
	if(inout > 1) goto read;

	param[10] = 1;		// element structure
	param[11] = versnr;		// Version Number

	if(np <= 0 && ne <= 0) return -2;

	fwrite(param,sizeof(int),12,pf);
	if(np > 0) // node-data
	{
		fwrite(nrp,sizeof(int),np+1,pf);
		fwrite(koor,sizeof(float),(np+1)*3,pf);
	}
	if(ne > 0 && nn1 > 0) // element-data
	{
		fwrite(nel,sizeof(int),(ne+1)*nn1,pf);
		fwrite(layerid,sizeof(int),ne+1,pf);
		fwrite(color,sizeof(int),ne+1,pf);
	}
	if(nla > 0)
	{
		fwrite(layeridd,sizeof(int),nla,pf);
		fwrite(layerixf,sizeof(int),nla,pf);
		if(nln > 0) fwrite(layername,1,nln,pf);
	}
	if(nkoors > 0) // koorsytems
		fwrite(koorsys,sizeof(float),(nkoors+1)*3*7,pf);
	if(nww > 0) // additional data
		fwrite(data,sizeof(float),nww,pf);

	fclose(pf);
	return 1;

read:
	nw = fread(param,sizeof(int),12,pf);
	if(nw != 12) goto fehler;

	nvers = param[11];
	if(nvers != versnr)
	{
		fclose(pf);
#if defined(_MAKROSA)
		AfxMessageBox("Version-Number is not correct: File can not be read");
#else
		printf("\n***Version-Number is not correct: File can not be read");
#endif
		return -5;
	}
	if(param[10] != 1)
	{
#if defined(_MAKROSA)
		CString ss;
		ss.LoadString(IDS_STRING63530);
		fclose(pf);
		AfxMessageBox(ss);
#endif
		return -5;
	}
	if(inout == 3)
	{
		fclose(pf);
		return 1;
	}

	np = param[0];
	ne = param[1];
	nn1 = param[2];
	nkoors = param[3];
	nw = param[4];
	nla = param[5];
	nln = param[6];

	if(np > 0)
	{
		nw = fread(nrp,sizeof(int),np+1,pf);
		if(nw != np+1) goto fehler;
		nw = fread(koor,sizeof(float),(np+1)*3,pf);
		if(nw != 3*(np+1)) goto fehler;
	}
	if(ne > 0 && nn1 > 0)
	{
		nw = fread(nel,sizeof(int),(ne+1)*nn1,pf);
		if(nw != (ne+1)*nn1) goto fehler;
		if(nvers > 1)
		{
			nw = fread(layerid,sizeof(int),ne+1,pf);
			if(nw != ne+1) goto fehler;
			nw = fread(color,sizeof(int),ne+1,pf);
			if(nw != ne+1) goto fehler;
		}
	}
	if(nvers > 1 && nla > 0)
	{
		nw = fread(layeridd,sizeof(int),nla,pf);
		if(nw != nla) goto fehler;
		nw = fread(layerixf,sizeof(int),nla,pf);
		if(nw != nla) goto fehler;
		if(nln > 0)
		{
			nw = fread(layername,1,nln,pf);
			if(nw != nln) goto fehler;
		}
	}
	if(nkoors > 0)
	{
		nw = fread(koorsys,sizeof(float),(nkoors+1)*3*7,pf);
		if(nw != (nkoors+1)*3*7) goto fehler;
	}
	if(nww > 0)
	{
		nw = fread(data,1,nww,pf);
		if(nw != nww) goto fehler;
	}

	fclose(pf);
	return 1;

fehler:
	fclose(pf);
#if !defined(_MAKROSA)
	printf("error on reading the file\n  %s\n",file);
#endif
	return -1;
}

// write or read load data
// retw: 1: ok < 0: file not read or written 
int getLoaddata(int inout, // 1=write;2=read;3=get-param only for space allocation
	char *file,		// filetitel
	int param[30],	// params see below
	long kn_muster[],
	int knoten[],
	float knlast[],
	long el_muster[],
	int elnr[],
	float el_last[],
	long kr_muster[],
	int kr_kn[],
	double *material[],
	long qs_muster[],
	int knqs[],
	float qsww[],
	float kkr[])
{
	static versnr = 1;		// version of filestructure: witten to the file in param[11]
	int kn_nmuster = param[0];
	int nkn = param[1];
	int lmuster = param[2];
	int llast = param[3];
	int el_nmuster = param[4];
	int nelnr = param[5];
	int el_lmuster = param[6];
	int el_llast = param[7];
	int kr_nmuster = param[8];
	int nkrkn = param[9];
	int kr_lmuster = param[12];
	int n_material = param[13];
	int qs_nmuster = param[14];
	int nknqs = param[15];
	int qs_lmuster = param[16];
	int nwqs = param[17];
	int nkkr = param[18];
	int nw;

	FILE *pf;

	if(inout > 1)
		memset(param,0,30*sizeof(int));

	if(inout > 1)
		pf = fopen(file,"rb");
	else
		pf = fopen(file,"wb");

	if(!pf)
	{
#if !defined(_MAKROSA)
		printf("\n***file %s can not be opened\n",file);
#endif
		return -1;
	}

	if(inout > 1) goto read;

	param[10] = 3;		// load data
	param[11] = versnr;		// Version Number

	fwrite(param,sizeof(int),30,pf);
	// knotenlasten
	if(kn_nmuster)
	{
		fwrite(kn_muster,sizeof(int),lmuster*kn_nmuster,pf);
		fwrite(knoten,sizeof(int),nkn,pf);
		fwrite(knlast,sizeof(float),llast*kn_nmuster,pf);
	}
	// elementlasten
	if(el_nmuster)
	{
		fwrite(el_muster,sizeof(int),el_lmuster*el_nmuster,pf);
		fwrite(elnr,sizeof(int),nelnr,pf);
		fwrite(el_last,sizeof(float),el_llast*el_nmuster,pf);
		if(nkkr) fwrite(kkr,sizeof(float),nkkr*4,pf);
	}
	// randbedingungen
	if(kr_nmuster)
	{
		fwrite(kr_muster,sizeof(int),kr_lmuster*kr_nmuster,pf);
		fwrite(kr_kn,sizeof(int),nkrkn,pf);
	}
	// materialwerte
	if(n_material)
	{
		for(int k=0;k<n_material;k++)
		{
			fwrite(material[k],sizeof(double),110,pf);
		}
	}
	// querschnitte
	if(qs_nmuster)
	{
		fwrite(qs_muster,sizeof(int),qs_lmuster*qs_nmuster,pf);
		fwrite(knqs,sizeof(int),nknqs,pf);
		fwrite(qsww,sizeof(float),nwqs,pf);
	}

	fclose(pf);
	return 1;

read:
	
	nw = fread(param,sizeof(int),30,pf);
	if(nw != 30) goto fehler;

	if(param[11] != versnr)
	{
		fclose(pf);
#if defined(_MAKROSA)
		AfxMessageBox("Old Version of File can not be read");
#else
		printf("Old version of file can not be read");
#endif
		return -5;
	}

	if(param[10] != 3)
	{
#if defined(_MAKROSA)
		fclose(pf);
		AfxMessageBox("File does not containe load data");
#else
		printf("File does not containe load data");
#endif
		return -5;
	}

	if(inout == 3)
	{
		fclose(pf);
		return 1;
	}

	kn_nmuster = param[0];
	nkn = param[1];
	lmuster = param[2];
	llast = param[3];
	el_nmuster = param[4];
	nelnr = param[5];
	el_lmuster = param[6];
	el_llast = param[7];
	kr_nmuster = param[8];
	nkrkn = param[9];
	kr_lmuster = param[12];
	n_material = param[13];
	qs_nmuster = param[14];
	nknqs = param[15];
	qs_lmuster = param[16];
	nwqs = param[17];
	nkkr = param[18];

	// knotenlasten
	if(kn_nmuster)
	{
		nw = fread(kn_muster,sizeof(int),lmuster*kn_nmuster,pf);
		if(nw != lmuster*kn_nmuster) goto fehler;
		nw = fread(knoten,sizeof(int),nkn,pf);
		if(nw != nkn) goto fehler;
		nw = fread(knlast,sizeof(float),llast*kn_nmuster,pf);
		if(nw != llast*kn_nmuster) goto fehler;
	}
	// elementlasten
	if(el_nmuster)
	{
		nw = fread(el_muster,sizeof(int),el_lmuster*el_nmuster,pf);
		if(nw != el_lmuster*el_nmuster) goto fehler;
		nw = fread(elnr,sizeof(int),nelnr,pf);
		if(nw != nelnr) goto fehler;
		nw = fread(el_last,sizeof(float),el_llast*el_nmuster,pf);
		if(nw != el_llast*el_nmuster) goto fehler;
		if(nkkr)
		{
			nw = fread(kkr,sizeof(float),nkkr*4,pf);
			if(nw != nkkr*4) goto fehler;
		}
	}
	// randbedingungen
	if(kr_nmuster)
	{
		nw = fread(kr_muster,sizeof(int),kr_lmuster*kr_nmuster,pf);
		if(nw != kr_lmuster*kr_nmuster) goto fehler;
		nw = fread(kr_kn,sizeof(int),nkrkn,pf);
		if(nw != nkrkn) goto fehler;
	}
	// materialwerte
	if(n_material)
	{
		for(int k=0;k<n_material;k++)
		{
			nw = fread(material[k],sizeof(double),110,pf);
			if(nw != 110) goto fehler;
		}
	}
	// querschnitte
	if(qs_nmuster)
	{
		nw = fread(qs_muster,sizeof(int),qs_lmuster*qs_nmuster,pf);
		if(nw != qs_lmuster*qs_nmuster) goto fehler;
		nw = fread(knqs,sizeof(int),nknqs,pf);
		if(nw != nknqs) goto fehler;
		nw = fread(qsww,sizeof(float),nwqs,pf);
		if(nw != nwqs) goto fehler;
	}
	fclose(pf);
	return 1;

fehler:
	fclose(pf);
#if !defined(_MAKROSA)
	printf("error on reading the file\n  %s\n",file);
#endif
	return -1;
}

