#include "interface.h"
#include <stdio.h>
#include <time.h>
#include <string.h>
#include "getdata.h"


extern int geomonly;	// only geometry data

void writeNodeData(FILE *pf,StructData *structdata);
void writeElementData(FILE *pf,StructData *structdata);
void writeMaterialProperties(FILE *pf,StructData *structdata);
void writeElementProperties(FILE *pf,StructData *structdata);
void writeCoordinateFrames(FILE *pf,StructData *structdata);
void writeDistibutedLoads(FILE *pf,StructData *structdata);
void writeNodeForces(FILE *pf,StructData *structdata);

int patran(char *file,StructData *structdata,char *project,
	int geom,int mfile)
{
	FILE *pf = fopen(file,"w");
	if(!pf) return 1;
	// Title Card
	fprintf(pf,"25%8d%8d%8d\n",0,0,1);
	char *p = strrchr(project,'.');
	if(p && strlen(p) <= 4) *p = '\0';
	fprintf(pf,"%s\n",project);

	// Summary Data
	fprintf(pf,"26%8d%8d%8d%8d%8d%8d%8d%8d\n",0,0,1,structdata->np,structdata->ne,
		structdata->nm,structdata->nelp,structdata->ncoor);
	struct tm *newtime;
	long ltime;
	time( &ltime );
	newtime = gmtime( &ltime );
	char *date = asctime( newtime);
	fprintf(pf,"%s",date);

	// Node Data
	writeNodeData(pf,structdata);
	// Element Data
	writeElementData(pf,structdata);
	// coordinate frames
	writeCoordinateFrames(pf,structdata);
	// Material properties
	if(!geom)
	{
		if(mfile) // different file title
		{
			fclose(pf);
			char *p = strchr(file,'.');
			*p = '\0';
			strcat(file,".pprop");
			pf = fopen(file,"w");
		}
		writeMaterialProperties(pf,structdata);
		// Element Properties
		writeElementProperties(pf,structdata);
		if(mfile) // different file title
		{
			fclose(pf);
			char *p = strchr(file,'.');
			*p = '\0';
			strcat(file,".pload");
			pf = fopen(file,"w");
		}
		// Distributed loads
		writeDistibutedLoads(pf,structdata);
		// Node forces
		writeNodeForces(pf,structdata);
	}
	fclose(pf);
	return 0;
}
// Node data
void writeNodeData(FILE *pf,StructData *structdata)
{
	char gtype,nspc[10],pspc[10],ziff[] = "123456";
	int spc,np = structdata->np;
	NodeData *p = structdata->nodedata;

	for(int k=0;k<np;k++)
	{
		gtype = p[k].gtype;
		spc = p[k].pspc;
		if(!gtype) gtype = 'G';
		strcpy(pspc,"000000");
		if(spc)
		{
			sprintf(nspc,"%d",spc);
			for(int i=0;i<6;i++)
				if(strchr(nspc,ziff[i])) pspc[i] = '1';
		}
		fprintf(pf," 1%8d%8d%8d\n",p[k].id,0,2);
		fprintf(pf,"%16.7f%16.7f%16.7f\n",p[k].xyz[0],p[k].xyz[1],p[k].xyz[2]);
		fprintf(pf,"%1d%c%8d%8d%8d  %6s\n",0,gtype,p[k].ndf,p[k].config,
			p[k].cid,pspc);
	}
}
// Element data
void writeElementData(FILE *pf,StructData *structdata)
{
	int ne = structdata->ne;
	ElementData *p = structdata->elemdata;
	NodeData *pn = structdata->nodedata;
	int kc,n1=0,n2;
	float x=0.f,y=0.f,z=0.f;
	int ik,nodes;

	for(int k=0;k<ne;k++)
	{
		n2 = 0;
		n1 = p[k].nad;
		nodes = p[k].nodes;
		kc = 1 + (nodes+9)/10 + (n1+4)/5;
		x=0.f;y=0.f;z=0.f;
		if(p[k].shape == 2)
		{
			n2 = p[k].lnodes[2];
			if(n2)
			{
				x = pn[n2-1].xyz[0];
				y = pn[n2-1].xyz[1];
				z = pn[n2-1].xyz[2];
			} else
				x = y = z = 0.f;
		}
		fprintf(pf," 2%8d%8d%8d%8d%8d\n",p[k].id,p[k].shape,kc,n1,n2);
		fprintf(pf,"%8d%8d%8d%8d%16.7f%16.7f%16.7f\n",
			p[k].nodes,0,p[k].pid,0,x,y,z);
		for(int i=0;i<nodes;i++)
		{
			ik = p[k].lnodes[i];
			if(ik) ik = pn[ik-1].id;
			fprintf(pf,"%8d",ik);
			if(i%10 == 9 || i == nodes-1) fprintf(pf,"\n");
		}
		if(n1)
		{
			for(i=0;i<n1;i++)
			{
				fprintf(pf,"%16.7f",p[k].adata[i]);
				if(i%5 == 4 || i == n1-1) fprintf(pf,"\n");
			}
		}
	}
}
// Matrerial Properties
void writeMaterialProperties(FILE *pf,StructData *structdata)
{
	int nm = structdata->nm;
	MaterialProperties *p = structdata->mprop;

	for(int k=0;k<nm;k++)
	{
		int kc,n1;
		n1 = p[k].n1;
		kc = 20+(n1+4)/5;
		fprintf(pf," 3%8d%8d%8d%8d\n",p[k].id,p[k].type,kc,n1);
		for(int l=0;l<96;l++)
		{
			fprintf(pf,"%16.7g",p[k].data[l]);
			if(l%5 == 4) fprintf(pf,"\n");
		}
		fprintf(pf,"\n");
		if(n1)
		{
			for(l=0;l<n1;l++)
			{
				fprintf(pf,"%16.7g",p[k].adata[l]);
				if(l%5 == 4 || l == n1-1) fprintf(pf,"\n");
			}
		}
	}
}
// Element Properties
void writeElementProperties(FILE *pf,StructData *structdata)
{
	int kc;
	int nelp = structdata->nelp;
	if(!nelp) return;
	ElementProperties *p = structdata->elemprop;

	for(int k=0;k<nelp;k++)
	{
		int n4 = p[k].nv;
		kc = (n4 + 4) / 5;
		fprintf(pf," 4%8d%8d%8d%8d%8d%8d%8d\n",p[k].id,p[k].matid,kc,
			p[k].shape,p[k].nodes,p[k].config,n4);
		for(int l=0;l<n4;l++)
		{
			fprintf(pf,"%16.7f",p[k].data[l]);
			if(l%5 == 4 || l == n4-1) fprintf(pf,"\n");
		}
	}
	return;
}
// Coordinate Frames
void writeCoordinateFrames(FILE *pf,StructData *structdata)
{
	CoorSystem *p = structdata->coorsys;
	int ncoor = structdata->ncoor;

	float a[3],b[3],pkt[6][3];

	for(int nk=0;nk<ncoor;nk++)
	{
		for(int k=0;k<3;k++)
			memcpy(pkt[k],p[nk].coors[k],3*sizeof(float));
		for(int i=0;i<3;i++)
		{
			a[i] = pkt[1][i] - pkt[0][i];
			b[i] = pkt[2][i] - pkt[0][i];
		}
		float d = vlaengen(a);
		if(d < 1.e-5) continue;
		for(i=0;i<3;i++) a[i] /= d;
		memcpy(pkt[5],a,3*sizeof(float));
		d = a_kreuz_bn(a,b,pkt[4],1);
		if(d < 1.e-5) continue;
		d = a_kreuz_bn(pkt[4],pkt[5],pkt[3],1);

		fprintf(pf," 5%8d%8d%8d\n",p[nk].id,p[nk].ktyp,4);
		int l = 0;
		for(k=0;k<6;k++)
		{
			for(i=0;i<3;i++)
			{
				fprintf(pf,"%16.7e",pkt[k][i]);
				++l;
				if((l && l%5 == 0) || l == 18) fprintf(pf,"\n");
			}
		}
	}
}
// Distributed Loads
void writeDistibutedLoads(FILE *pf,StructData *structdata)
{
	DistributedLoads *p = structdata->elemloads;
	int nel = structdata->nel;
	char icomp[10],comp[10],node[10],ziff[] = "12345678";
	int nicomp,nnode;
	
	for(int k=0;k<nel;k++)
	{
		int kc = 1 + (p[k].ndw+4)/5;
		int ndw = p[k].ndw;
		strcpy(icomp,"000000");
		nicomp = p[k].icomp;
		if(nicomp)
		{
			sprintf(comp,"%d",nicomp);
			for(int i=0;i<6;i++)
				if(strchr(comp,ziff[i])) icomp[i] = '1';
		}
		strcpy(node,"00000000");
		nnode = p[k].node;
		if(nnode)
		{
			sprintf(comp,"%d",nnode);
			for(int i=0;i<8;i++)
				if(strchr(comp,ziff[i])) node[i] = '1';
		}
		fprintf(pf," 6%8d%8d%8d\n",p[k].id,p[k].loadset,kc);
		fprintf(pf,"%1d%1d%1d%6s%8s%2d\n",p[k].ltyp,p[k].eflag,p[k].gflag,
			icomp,node,p[k].nfe);
		for(int l=0;l<ndw;l++)
		{
			fprintf(pf,"%16.7f",p[k].pdata[l]);
			if(l%5 == 4 || l == ndw-1) fprintf(pf,"\n");
		}
	}
}
// Node Forces
void writeNodeForces(FILE *pf,StructData *structdata)
{
	char icomp[7];
	float fdata[3];
	NodeForces *p = structdata->nodeforces;
	int nf = structdata->nf;

	for(int k=0;k<nf;k++)
	{
		int type = p[k].type;
		memcpy(fdata,p[k].fdata,3*sizeof(float));
		if(!type)
		{
			strcpy(icomp,"111000");
		} else if(type == 1)
		{
			strcpy(icomp,"000111");
		} else
			continue;

		int kc = 2;
		fprintf(pf," 7%8d%8d%8d\n",p[k].id,p[k].loadset,kc);
		fprintf(pf,"%8d%6s\n",p[k].cid,icomp);
		for(int l=0;l<3;l++)
		{
			fprintf(pf,"%16.7f",fdata[l]);
		}
		fprintf(pf,"\n");
	}
}
